@@ -244,7 +244,7 @@ wnck_selector_set_active_window (WnckSelector *selector, WnckWindow *window)
 static void
 wnck_selector_make_menu_consistent (WnckSelector *selector)
 {
-  GList     *l;
+  GList     *l, *children;
   int        workspace_n;
   GtkWidget *workspace_item;
   GtkWidget *separator;
@@ -261,7 +261,9 @@ wnck_selector_make_menu_consistent (WnckSelector *selector)
 
   visible_window = FALSE;
 
-  for (l = GTK_MENU_SHELL (selector->priv->menu)->children; l; l = l->next)
+  children = gtk_container_get_children (GTK_CONTAINER (selector->priv->menu));
+
+  for (l = children; l; l = l->next)
     {
       int i;
 
@@ -315,6 +317,8 @@ wnck_selector_make_menu_consistent (WnckSelector *selector)
         } /* end if (normal item) */
     }
 
+  g_list_free (children);
+
   /* do we have a trailing workspace item to be hidden? */
   if (workspace_item)
     gtk_widget_hide (workspace_item);
@@ -505,6 +509,7 @@ wnck_selector_activate_window (WnckWindow *window)
 static gint
 wnck_selector_get_width (GtkWidget *widget, const char *text)
 {
+  GtkStyle *style;
   PangoContext *context;
   PangoFontMetrics *metrics;
   gint char_width;
@@ -515,9 +520,10 @@ wnck_selector_get_width (GtkWidget *widget, const char *text)
   gint width;
 
   gtk_widget_ensure_style (widget);
+  style = gtk_widget_get_style (widget);
 
   context = gtk_widget_get_pango_context (widget);
-  metrics = pango_context_get_metrics (context, widget->style->font_desc,
+  metrics = pango_context_get_metrics (context, style->font_desc,
                                        pango_context_get_language (context));
   char_width = pango_font_metrics_get_approximate_char_width (metrics);
   pango_font_metrics_unref (metrics);
@@ -548,7 +554,7 @@ wnck_selector_drag_begin (GtkWidget          *widget,
       if (GTK_IS_MENU (widget))
         widget = gtk_menu_get_attach_widget (GTK_MENU (widget));
       else
-        widget = widget->parent;
+        widget = gtk_widget_get_parent (widget);
     }
 
   if (widget)
@@ -567,7 +573,7 @@ wnck_selector_drag_data_get (GtkWidget          *widget,
 
   xid = wnck_window_get_xid (window);
   gtk_selection_data_set (selection_data,
- 		          selection_data->target,
+                          gtk_selection_data_get_target (selection_data),
 			  8, (guchar *)&xid, sizeof (gulong));
 }
 
@@ -631,24 +637,17 @@ wnck_selector_item_new (WnckSelector *selector,
   return item;
 }
 
-static gboolean
-wnck_selector_workspace_label_exposed (GtkWidget *widget)
-{
-  /* Bad hack to make the label draw normally, instead of insensitive. */
-  widget->state = GTK_STATE_NORMAL;
-
-  return FALSE;
-}
-
 static void
 wnck_selector_workspace_name_changed (WnckWorkspace *workspace,
                                       GtkLabel      *label)
 {
+  GtkStyle *style;
   GdkColor *color;
   char     *name;
   char     *markup;
 
-  color = &GTK_WIDGET (label)->style->fg[GTK_STATE_INSENSITIVE];
+  style = gtk_widget_get_style (GTK_WIDGET (label));
+  color = &style->fg[GTK_STATE_INSENSITIVE];
 
   name = g_markup_escape_text (wnck_workspace_get_name (workspace), -1);
   markup = g_strdup_printf ("<span size=\"x-small\" style=\"italic\" foreground=\"#%.2x%.2x%.2x\">%s</span>",
@@ -678,14 +677,13 @@ wnck_selector_add_workspace (WnckSelector *selector,
 
   workspace = wnck_screen_get_workspace (screen, workspace_n);
 
-  item = gtk_menu_item_new ();
-  gtk_widget_set_sensitive (item, FALSE);
+  /* We use a separator in which we add a label. This makes the menu item not
+   * selectable without any hack. */
+  item = gtk_separator_menu_item_new ();
 
   label = gtk_label_new ("");
   gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
   gtk_widget_show (label);
-  g_signal_connect (G_OBJECT (label), "expose-event",
-                    G_CALLBACK (wnck_selector_workspace_label_exposed), NULL);
   /* the handler will also take care of setting the name for the first time,
    * and we'll be able to adapt to theme changes */
   g_signal_connect (G_OBJECT (label), "style-set",
@@ -761,16 +759,18 @@ wnck_selector_insert_window (WnckSelector *selector, WnckWindow *window)
     {
       /* window is pinned or in the current workspace
        * => insert before the separator */
-      GList *l;
+      GList *l, *children;
 
       i = 0;
 
-      for (l = GTK_MENU_SHELL (selector->priv->menu)->children; l; l = l->next)
+      children = gtk_container_get_children (GTK_CONTAINER (selector->priv->menu));
+      for (l = children; l; l = l->next)
         {
           if (GTK_IS_SEPARATOR_MENU_ITEM (l->data))
             break;
           i++;
         }
+      g_list_free (children);
 
       gtk_menu_shell_insert (GTK_MENU_SHELL (selector->priv->menu),
                              item, i);
@@ -785,12 +785,12 @@ wnck_selector_insert_window (WnckSelector *selector, WnckWindow *window)
       else
         {
           /* insert just before the next workspace item */
-          GList *l;
+          GList *l, *children;
 
           i = 0;
 
-          for (l = GTK_MENU_SHELL (selector->priv->menu)->children;
-               l; l = l->next)
+          children = gtk_container_get_children (GTK_CONTAINER (selector->priv->menu));
+          for (l = children; l; l = l->next)
             {
               int j;
               j = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (l->data),
@@ -799,6 +799,7 @@ wnck_selector_insert_window (WnckSelector *selector, WnckWindow *window)
                 break;
               i++;
             }
+          g_list_free (children);
 
           gtk_menu_shell_insert (GTK_MENU_SHELL (selector->priv->menu),
                                  item, i);
@@ -884,7 +885,7 @@ wnck_selector_workspace_destroyed (WnckScreen    *screen,
                                    WnckWorkspace *workspace,
                                    WnckSelector  *selector)
 {
-  GList     *l;
+  GList     *l, *children;
   GtkWidget *destroy;
   int        i;
 
@@ -896,7 +897,9 @@ wnck_selector_workspace_destroyed (WnckScreen    *screen,
   i = wnck_workspace_get_number (workspace);
 
   /* search for the item of this workspace so that we destroy it */
-  for (l = GTK_MENU_SHELL (selector->priv->menu)->children; l; l = l->next)
+  children = gtk_container_get_children (GTK_CONTAINER (selector->priv->menu));
+
+  for (l = children; l; l = l->next)
     {
       int j;
 
@@ -912,6 +915,8 @@ wnck_selector_workspace_destroyed (WnckScreen    *screen,
                            GINT_TO_POINTER (j - 1));
     }
 
+  g_list_free (children);
+
   if (destroy)
     gtk_widget_destroy (destroy);
 
